import 'dart:collection';
import 'dart:io';

import 'package:dio/dio.dart';
import 'package:_private_core/_private_core.dart';
import '../auth/models/check_email_result.dart';
import '../common/models/file_upload_result.dart';
import '../model/info_by_username.dart';
import '../model/username_available_result.dart';
import '../resources.dart';

class AppApiImp extends AppApi {
  Map<String, dynamic> _mapParms([Map? value]) {
    Map<String, dynamic> params = {};
    if (value != null && value.isNotEmpty) {
      params.addAll(HashMap.from(value));
    }
    params
      ..removeWhere((key, value) => value == null)
      ..addAll({
        'platformId': valuePlatformByConfig(),
      });
    return params;
  }

  @override
  Future<NetworkResponse> updateRegistrationLevelForUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateRegistrationLevelForUser(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getAccountOwnershipFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.getAccountOwnershipFromPro(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> checkUsernameAvailable(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.checkUsernameAvailable(),
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => UsernameAvailableResult.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> refreshUserToken(refreshToken) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response =
          await AppClient(requiredToken: true, token: refreshToken)
              .post(AppEndpoint.refreshUserToken());
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> refreshProToken(refreshToken) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response =
          await AppClient(requiredToken: true, token: refreshToken)
              .post(AppEndpoint.refreshProToken());
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> checkNewPhoneFromUser(phone) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response =
          await AppClient(requiredToken: false, enableErrorHandler: false).get(
              AppEndpoint.checkNewPhoneFromUser(),
              queryParameters: {'phone': phone});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkNewPhoneFromPro(phone) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response =
          await AppClient(requiredToken: false, enableErrorHandler: false).get(
              AppEndpoint.checkNewPhoneFromPro(),
              queryParameters: {'phone': phone});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkExistPhoneFromUser(phone) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.checkExistPhoneFromUser(),
          queryParameters: {'phone': phone});
      return NetworkResponse.fromResponse(response,
          converter: (json) => UserModel.fromJson(json), prefix: '');
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addNotificationToken(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addNotificationToken(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createOrLoginPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .post(AppEndpoint.createOrLoginPro(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateNewUserPasswordByFirebase(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).post(
          AppEndpoint.updateNewUserPasswordByFirebase(),
          data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginUserByPhone(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).post(
        AppEndpoint.loginUserByPhone(),
        data: _mapParms(params),
      );
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginUserByPhoneAndFirebase(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).post(
        AppEndpoint.loginUserByPhoneAndFirebase(),
        data: _mapParms(params),
      );
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginUserByEmail(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .post(AppEndpoint.loginUserByEmail(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createOrLoginUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .post(AppEndpoint.createOrLoginUser(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createNewUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .post(AppEndpoint.createNewUser(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createNewFirebaseCustomToken() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(
        AppEndpoint.createNewFirebaseCustomToken(),
      );
      return NetworkResponse.fromResponse(response, converter: (json) => json);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }


  @override
  Future<NetworkResponse> getMqttInfo() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.getMqttInfo());
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLoginMqtt.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .put(AppEndpoint.updateUser(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: UpdateInfoResult(status: response.statusCode == 200));
    } on DioException catch (e) {
      if (e.response?.data != null && e.response!.data['error'] is String) {
        return NetworkResponse.fromResponse(e.response!,
            value: UpdateInfoResult(
                status: false, msg: e.response!.data['error']));
      }
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updatePro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .put(AppEndpoint.updatePro(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: UpdateInfoResult(status: true));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateProFromSA(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .put(AppEndpoint.updateProFromSA(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateUserFromSA(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .put(AppEndpoint.updateUserFromSA(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> logoutPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.logoutPro(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> logoutUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.logoutUser(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  } 

  @override
  Future<NetworkResponse> updateProPwd(
      String oldPassword, String newPassword) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .put(AppEndpoint.updateProPwd(),
              data: {'oldPassword': oldPassword, 'newPassword': newPassword});
      return NetworkResponse.fromResponse(response,
          value: UpdateInfoResult(status: response.statusCode == 200));
    } on DioException catch (e) {
      if (e.response?.data != null && e.response!.data['error'] is String) {
        return NetworkResponse.fromResponse(e.response!,
            value: UpdateInfoResult(
                status: false, msg: e.response!.data['error']));
      }
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateUserPwd(
      String oldPassword, String newPassword) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .put(AppEndpoint.updateUserPwd(),
              data: {'oldPassword': oldPassword, 'newPassword': newPassword});
      return NetworkResponse.fromResponse(response,
          value: UpdateInfoResult(status: response.statusCode == 200));
    } on DioException catch (e) {
      if (e.response?.data != null && e.response!.data['error'] is String) {
        return NetworkResponse.fromResponse(e.response!,
            value: UpdateInfoResult(
                status: false, msg: e.response!.data['error']));
      }
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkUpdateEmailFromUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.checkUpdateEmailFromUser(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: UpdateInfoResult(status: response.statusCode == 200));
    } on DioException catch (e) {
      if (e.response?.data != null && e.response!.data['error'] is String) {
        return NetworkResponse.fromResponse(e.response!,
            value: UpdateInfoResult(
                status: false, msg: e.response!.data['error']));
      }
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> confirmUpdateUserEmailCode(email, code) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.confirmUpdateUserEmailCode(),
              data: {'email': email, 'code': code});
      return NetworkResponse.fromResponse(response,
          converter: (json) => ConfirmEmailCodeResponse.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateUserEmail(_token) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .put(AppEndpoint.updateUserEmail(), data: {'token': _token});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkUpdateEmailFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.checkUpdateEmailFromPro(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: UpdateInfoResult(status: response.statusCode == 200));
    } on DioException catch (e) {
      if (e.response?.data != null && e.response!.data['error'] is String) {
        return NetworkResponse.fromResponse(e.response!,
            value: UpdateInfoResult(
                status: false, msg: e.response!.data['error']));
      }
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> confirmUpdateProEmailCode(email, code) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.confirmUpdateProEmailCode(),
              data: {'email': email, 'code': code});
      return NetworkResponse.fromResponse(response,
          converter: (json) => ConfirmEmailCodeResponse.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateProEmail(_token) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .put(AppEndpoint.updateProEmail(), data: {'token': _token});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkExistEmailFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.checkExistEmailFromPro(),
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: '', converter: (json) => CheckEmailResult.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkNewProEmail(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .post(AppEndpoint.checkNewProEmail(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: '', converter: (json) => CheckEmailResult.fromJson(json));
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkIfEmailExist(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .post(AppEndpoint.checkIfEmailExist(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: '', converter: (json) => CheckEmailResult.fromJson(json));
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkNewEmailUserEmailConfirm(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).post(
          AppEndpoint.checkNewEmailUserEmailConfirm(),
          data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => ConfirmOtp.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkEmailPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .post(AppEndpoint.checkEmailPro(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: '', converter: (json) => CheckEmailResult.fromJson(json));
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> confirmNewProCode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .post(AppEndpoint.confirmNewProCode(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => ConfirmOtp.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }


  @override
  Future<NetworkResponse> createNewPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .post(AppEndpoint.createNewPro(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCategoriesList() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.getCategoriesList(),
          queryParameters: {'catLanguageCode': null});
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json['maincats'] as List).map((e) => MainCat.fromJson(e)).toList(),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .post(AppEndpoint.loginPro(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginUserOrProByUsername(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).post(
          AppEndpoint.loginUserOrProByUsername(),
          data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginProByPhoneAndFirebase(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).post(
          AppEndpoint.loginProByPhoneAndFirebase(),
          data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginProCrmByQrCode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .post(AppEndpoint.loginProCrmByQrCode(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  } 

  @override
  Future<NetworkResponse> getInfoByUsername(username) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.getInfoByUsername(),
          queryParameters: {'username': username});
      return NetworkResponse.fromResponse(response,
          converter: (json) => InfoByUsername.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkLoginProByEmailOTP(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .post(AppEndpoint.checkLoginProByEmailOTP(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginProByEmailOTPConfirm(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).post(
          AppEndpoint.loginProByEmailOTPConfirm(),
          data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAssetsForProFromUniversalDashboard(
      proUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.getAssetsForProFromUniversalDashboard(),
          queryParameters: {'proUniqueId': proUniqueId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => OrgProInfo.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse<List<DateFormatModel>>> getDateFormatList() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .get(AppEndpoint.getDateFormatList());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<DateFormatModel>((i) => DateFormatModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse<List<TimeFormatModel>>> getTimeFormatList() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .get(AppEndpoint.getTimeFormatList());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<TimeFormatModel>((i) => TimeFormatModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse<List<IndustryModel>>> getIndustriesList() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.getIndustriesList(),
          queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<IndustryModel>((i) => IndustryModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCurrencies() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .get(AppEndpoint.getCurrencies(), queryParameters: {'available': 1});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<CurrencyAvailable>((i) => CurrencyAvailable.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getTerminalsFromPro(terminalType) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getTerminalsFromPro(),
              queryParameters: {'terminalType': terminalType});
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => Terminal.fromJson(e)).toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllTerminalFromAdmin(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAllTerminalFromAdmin(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => Terminal.fromJson(e)).toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_getCurrentTerminalsOfPlaylistFromPro(
      params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.v2_getCurrentTerminalsOfPlaylistFromPro(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => Terminal.fromJson(e)).toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getVideoConfigFromPlayerVimeo(vimeoId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.getVideoConfigFromPlayerVimeo(),
          queryParameters: {'videoId': vimeoId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => VimeoConfig.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateTerminalFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateTerminalFromPro(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> sendHomeToTerminal(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.sendHomeToTerminal(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> removeHomeFromTerminal(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.removeHomeFromTerminal(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateProduct(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateProduct(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> takeMissionFromSeller(missionId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.takeMissionFromSeller(),
              data: {'missionId': missionId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getProductCategories() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getProductCategories(),
              queryParameters: {'display': 1});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ProductCategory>((i) => ProductCategory.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getProductsFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getProductsFromPro(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ProductModel>((i) => ProductModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getProductOfProFromSA(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getProductOfProFromSA(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ProductModel>((i) => ProductModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getProductOfUserFromSA(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getProductOfUserFromSA(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ProductModel>((i) => ProductModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getStatusList() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getStatusList());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ProductStatus>((i) => ProductStatus.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllPublicEventsFromSearch(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.getAllPublicEventsFromSearch(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PublishEvent>((i) => PublishEvent.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllPublicEventsAroundMe(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.getAllPublicEventsAroundMe(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PublishEvent>((i) => PublishEvent.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPublicEventsListFromUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.getPublicEventsListFromUser(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PublishEvent>((i) => PublishEvent.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getSuggestionEventsFromUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.getSuggestionEventsFromUser(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PublishEvent>((i) => PublishEvent.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPublicEventsCategories(languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.getPublicEventsCategories(),
              queryParameters: {'languageCode': languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PublishEventCategory>(
                  (i) => PublishEventCategory.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getFavoritePublicEventsCategoriesFromUser(
      languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.getFavoritePublicEventsCategoriesFromUser(),
              queryParameters: {'languageCode': languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PublishEventCategory>(
                  (i) => PublishEventCategory.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getLikedPublicEventsFromUser() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.getLikedPublicEventsFromUser());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PublishEvent>((i) => PublishEvent.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getTicketListFromPublicEvent(publicEventId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.getTicketListFromPublicEvent(),
              queryParameters: {'publicEventId': publicEventId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<TicketInfoModel>((i) => TicketInfoModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addUserLikeToPublicEvent(publicEventId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.addUserLikeToPublicEvent(),
              data: {'publicEventId': publicEventId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> setFavoriteCatForUser(catId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.setFavoriteCatForUser(), data: {'catId': catId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> removeFavoriteCatForUser(catId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .delete(AppEndpoint.removeFavoriteCatForUser(),
              data: {'catId': catId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addSeenSuggestionEventFromUser(
      publicEventId, utcModifier) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.addSeenSuggestionEventFromUser(), data: {
        'publicEventId': publicEventId,
        'utcModifier': utcModifier
      });
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> bookFreeTicketForPublicEvents(
      arckipelTicketId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.bookFreeTicketForPublicEvents(),
              data: {'arckipelTicketId': arckipelTicketId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllowedFeaturesForOowooApp() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.getAllowedFeaturesForOowooApp());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<AllowedFeatureModel>((i) => AllowedFeatureModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_getDashboardConfigFromPro() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.v2_getDashboardConfigFromPro(),
              queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => DashboardConfig.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createCustomerCardStripe(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.createCustomerCardStripe(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => PaymentCardStripe.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> deleteCustomerCardStripe(cardId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .delete(AppEndpoint.deleteCustomerCardStripe(),
              data: {'cardId': cardId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCustomerCardsStripe(page, pageSize) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.getCustomerCardsStripe(),
              queryParameters: {'page': page, 'page_size': pageSize});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PaymentCardStripe>((i) => PaymentCardStripe.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> setCustomerCardDefaultStripe(cardId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .put(AppEndpoint.setCustomerCardDefaultStripe(),
              data: {'cardId': cardId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> becomeSeller(confirmed) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.becomeSeller(), data: {'confirmed': confirmed});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateImageForBecomeSeller(fileType, path,
      [ProgressCallback? callback]) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      String fileName = path.split('/').last;
      FormData formData = FormData.fromMap({
        "fileType": fileType,
        "fileName": MultipartFile.fromBytes(File(path).readAsBytesSync(),
            filename: fileName),
      });
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.updateImageForBecomeSeller(),
              data: formData, onSendProgress: callback);
      return NetworkResponse.fromResponse(response,
          value: response.data['values']['fileName']);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> quitSellerProgram(temporary) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.quitSellerProgram(),
              data: {'temporary': temporary});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> recoverySellerProgram() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.recoverySellerProgram());
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getTncOowooApp() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.getTncOowooApp(),
          queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['text']);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getTncOowooSeller() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.getTncOowooSeller(),
          queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['text']);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> birthsOnThisDay(date) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .get('https://byabbe.se/on-this-day/$date/births.json');
      return NetworkResponse.fromResponse(response,
          prefix: 'births',
          converter: (json) => (json as List<dynamic>)
              .map<OnThisDayModel>((i) => OnThisDayModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> deathsOnThisDay(date) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .get('https://byabbe.se/on-this-day/$date/deaths.json');
      return NetworkResponse.fromResponse(response,
          prefix: 'deaths',
          converter: (json) => (json as List<dynamic>)
              .map<OnThisDayModel>((i) => OnThisDayModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> eventsOnThisDay(date) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .get('https://byabbe.se/on-this-day/$date/events.json');
      return NetworkResponse.fromResponse(response,
          prefix: 'events',
          converter: (json) => (json as List<dynamic>)
              .map<OnThisDayModel>((i) => OnThisDayModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> pexelsSearch(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true,
              isAuthorizationCustom: true,
              token: PEXEL_API_KEY)
          .get('https://api.pexels.com/v1/search',
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: 'photos',
          converter: (json) => (json as List<dynamic>)
              .map<PexelPhotoModel>((i) => PexelPhotoModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> weatherForecast(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.weatherapi.com/v1/forecast.json',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: 'forecast',
          converter: (json) => WeatherForecast.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> weatherHistory(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.weatherapi.com/v1/history.json',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: 'forecast',
          converter: (json) => WeatherForecast.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getMissionsFromSeller(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getMissionsFromSeller(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<SellerMissionModel>((i) => SellerMissionModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getMissionTypes(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getMissionTypes(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<MissionTypeModel>((i) => MissionTypeModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOoBRPlaceInfoById(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOoBRPlaceInfoById(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => BrPlaceModel.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOoBRPlacesAroundMe(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOoBRPlacesAroundMe(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPlaceModel>((i) => BrPlaceModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOoBRPlacesByFilterValues(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOoBRPlacesByFilterValues(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPlaceModel>((i) => BrPlaceModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getFavoriteOobrPlaces() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getFavoriteOobrPlaces());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPlaceModel>((i) => BrPlaceModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateFilterValueForOobr(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.updateFilterValueForOobr(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['updateFilterValueForOobr']);
    } on DioException catch (e) {
      if (e.response != null &&
          e.response!.statusCode == 400 &&
          e.response!.data != null) {
        return NetworkResponse.fromResponse(e.response!,
            value: e.response!.data['error']);
      }
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPromotedVideosForBar(oobrPlaceId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPromotedVideosForBar(),
              queryParameters: {'oobrPlaceId': oobrPlaceId});
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List<dynamic>).map<String>((i) => i).toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPromotedVideosForOobrPlaces(lat, lng) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPromotedVideosForOobrPlaces(),
              queryParameters: {'lat': lat, 'lng': lng});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPromotedVideoModel>(
                  (i) => BrPromotedVideoModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPromotedVideosForOoh(lat, lng) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPromotedVideosForOoh(),
              queryParameters: {'lat': lat, 'lng': lng});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPromotedVideoModel>(
                  (i) => BrPromotedVideoModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOoBRCategories(languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOoBRCategories(),
              queryParameters: {'languageCode': languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPlaceCateModel>((i) => BrPlaceCateModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOoBRFilters(languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOoBRFilters(),
              queryParameters: {'languageCode': languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPlaceFilterModel>((i) => BrPlaceFilterModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkBarGooglePlaceIdByContributor(
      googlePlaceId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.checkBarGooglePlaceIdByContributor(),
              queryParameters: {'googlePlaceId': googlePlaceId});
      return NetworkResponse.fromResponse(response, value: '');
    } on DioException catch (e) {
      if (e.response != null &&
          e.response!.statusCode == 400 &&
          e.response!.data != null) {
        return NetworkResponse.fromResponse(e.response!,
            value: e.response!.data['oobrPlaceId']);
      }
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createNewOobrPlaceByContributor(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createNewOobrPlaceByContributor(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addOobrPlaceToFavorites(oobrPlaceId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addOobrPlaceToFavorites(),
              data: {'oobrPlaceId': oobrPlaceId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOobrPlaceSchedules(oobrPlaceId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOobrPlaceSchedules(),
              queryParameters: {'oobrPlaceId': oobrPlaceId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPlaceScheduleModel>(
                  (i) => BrPlaceScheduleModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOobrPlaceMenuList(
      oobrPlaceId, languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOobrPlaceMenuList(), queryParameters: {
        'oobrPlaceId': oobrPlaceId,
        'languageCode': languageCode
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPlaceMenu>((i) => BrPlaceMenu.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOobrPlaceMenuContent(
      oobrPlaceId, languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOobrPlaceMenuContent(), queryParameters: {
        'oobrPlaceId': oobrPlaceId,
        'languageCode': languageCode
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => BrPlaceMenuContent.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOptionsForItem(itemId, languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOptionsForItem(), queryParameters: {
        'itemId': itemId,
        'languageCode': languageCode
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PlaceMenuItemOption>((i) => PlaceMenuItemOption.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllergensForItem(itemId, languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAllergensForItem(), queryParameters: {
        'itemId': itemId,
        'languageCode': languageCode
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PlaceMenuAllergens>((i) => PlaceMenuAllergens.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getContactsList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getContactsList(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ContactModel>((i) => ContactModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addUserInContactsList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addUserInContactsList(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => ContactModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> addUserOrProToContactListByFirebaseId(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addUserOrProToContactListByFirebaseId(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => ContactModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> createFirebaseDynamicLink(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).post(
          AppEndpoint.createFirebaseDynamicLink(),
          data: _mapParms(params));
      return NetworkResponse.fromResponse(response, converter: (json) => json);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> deleteContact(id) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.deleteContact(), data: {'id': id});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> updateCommentForContact(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateCommentForContact(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getExistingUserByPhone(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getExistingUserByPhone(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => ContactModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> createOrderForOobrPlace(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createOrderForOobrPlace(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<UserModel>((i) => UserModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAnythingFromQr(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAnythingFromQr(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => QrDataModel.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getDetailOobrInvitations(oobrEventId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getDetailOobrInvitations(),
              queryParameters: {'oobrEventId': oobrEventId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => BrPlaceInviteModel.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> inviteContactsToOobrPlace(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.inviteContactsToOobrPlace(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> answerOobrInvitation(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.answerOobrInvitation(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getEmergencyInfos(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getEmergencyInfos(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => EmergencyResult.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> sendEmergencyRequest(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.sendEmergencyRequest(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createSharedPcEvent(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createSharedPcEvent(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['id']);
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPcEventFromUser(pcEventId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPcEventFromUser(),
              queryParameters: {'pcEventId': pcEventId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => PeEventModel.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> setThumbnailForPcEvent(path, pcEventId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      String fileName = path.split('/').last;
      FormData data = FormData.fromMap({
        'pcEventId': pcEventId,
        "fileName": MultipartFile.fromBytes(File(path).readAsBytesSync(),
            filename: fileName),
      });
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.setThumbnailForPcEvent(), data: data);
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['fileName']);
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> shareCalendarToUsers(params) =>
      throw UnimplementedError();

  @override
  Future<NetworkResponse> whatsNext(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.whatsNext(), queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => WhatNextModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> generateToken(channelName, expiredTime) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.generateVideoCallToken(),
              data: {'channel': channelName});
      return NetworkResponse.fromResponse(response,
          converter: (json) => AgoraToken.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getNumberOfSalesFromSellers(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getNumberOfSalesFromSellers(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => NumberOfSaleModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> setBankAccountFromUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.setBankAccountFromUser(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => BankAccountModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> updateBankInfosFromSeller(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateBankInfosFromSeller(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => BankAccountModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getAccountTypes() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAccountTypes());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => BankAccountTypeModel.fromJson(e))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getBankInfosFromSeller() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getBankInfosFromSeller());
      return NetworkResponse.fromResponse(response,
          converter: (json) => BankAccountModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> addUserOrProToTrackingPeople(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addUserOrProToTrackingPeople(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => TrackingInfoModel.fromJson(e))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getTrackingInfosForMeeting(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getTrackingInfosForMeeting(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => TrackingInfoModel.fromJson(e))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> updateTrackingInfosForMeeting(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateTrackingInfosForMeeting(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => TrackingInfoModel.fromJson(e))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getLearnMoreTrackingVideo() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.getLearnMoreTrackingVideo(),
          queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response, converter: (json) => json);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getReminderTypes() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getReminderTypes(),
              queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => ReminderType.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOowooMoveProperties(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOowooMoveProperties(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => MoveModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOowooVideoByTypeId(typeId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOowooVideoByTypeId(), queryParameters: {
        'typeId': typeId,
        'languageCode': appPrefs.languageCode
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => VideoUrl.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCalmByCat(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getCalmByCat(), queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<CalmModel>((i) => CalmModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCalmCategories() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getCalmCategories(),
              queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<CalmCate>((i) => CalmCate.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getSubscribersForCalmId(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getSubscribersForCalmId(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json['subscribers'] as List<dynamic>)
              .map<CalmSubscriber>((i) => CalmSubscriber.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> subscribeToCalmId(calmId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.subscribeToCalmId(), data: {'calmId': calmId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> testSendNotificationToPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.testSendNotificationToPro(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> testSendNotificationToUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.testSendNotificationToUser(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> notifVideoCallProFromUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.notifVideoCallProFromUser(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> notifVideoCallUserFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.notifVideoCallUserFromPro(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getReviewsFromPro(proId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getReviewsFromPro(),
              queryParameters: {'proId': proId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ReviewModel>((i) => ReviewModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> cancelTicketByUser(
      ticketUniqueId, status, comment) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.cancelTicketByUser(),
              data: {'ticketUniqueId': ticketUniqueId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> confirmTicketByUser(
      ticketUniqueId, status, comment) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.confirmTicketByUser(), data: {
        'ticketUniqueId': ticketUniqueId,
        'status': status,
        'comment': comment
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => TicketInfoModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> deleteEventsFromUser(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.deleteEventsFromUser(), data: {
        'events': [ticketUniqueId]
      });
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllEventsFromUser() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAllEventsFromUser(),
              queryParameters: {'catLanguageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<EventInfoModel>((i) => EventInfoModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getEventInfoFromUser(eventUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getEventInfoFromUser(), queryParameters: {
        'catLanguageCode': appPrefs.languageCode,
        'eventUniqueId': eventUniqueId
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => EventInfoModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getQueueInfoFromUser(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getQueueInfoFromUser(), queryParameters: {
        'catLanguageCode': appPrefs.languageCode,
        'ticketUniqueId': ticketUniqueId
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => EventQueueModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> ticketUserCallBack(ticketUniqueId, duration) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.ticketUserCallBack(),
              data: {'ticketUniqueId': ticketUniqueId, 'duration': duration});
      return NetworkResponse.fromResponse(response,
          converter: (json) => TicketInfoModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getUserInfoByFirebaseId(firebaseId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getUserInfoByFirebaseId(),
              queryParameters: {'firebaseId': firebaseId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => UserModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getProInfoByFirebaseId(firebaseId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getProInfoByFirebaseId(),
              queryParameters: {'firebaseId': firebaseId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => ProsModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addToFavourite(proId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addToFavourite(), data: {'proId': proId});
      return NetworkResponse.fromResponse(response, value: true);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getFavouriteForUser() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getFavouriteForUser());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ProsModel>((i) => ProsModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> removeFromFavourite(proId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.removeFromFavourite(), data: {'proId': proId});
      return NetworkResponse.fromResponse(response, value: true);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPcEventFromPro(pcEventId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPcEventFromPro(),
              queryParameters: {'pcEventId': pcEventId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => PeEventModel.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> deleteCodeFromLog(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.deleteCodeFromLog(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> createCode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createCode(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => DigicodeCodeModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> updateLiveCode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateLiveCode(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> shareCodeDigicode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.shareCodeDigicode(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> deleteLiveCode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.deleteLiveCode(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getCodesFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getCodesFromPro(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => DigicodeCodeModel.fromJson(e))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getDigicodesFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getDigicodesFromPro(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => DigicodeModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getDigicodeById(id) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getDigicodeById(), queryParameters: {'id': id});
      return NetworkResponse.fromResponse(response,
          converter: (json) => DigicodeModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> checkDigicodeBySerialAndToken(
      productSerial, authcode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              options: BaseOptions(headers: {'auth-code': authcode}),
              requiredToken: true,
              token: await appPrefs.getNormalToken())
          .get(AppEndpoint.checkDigicodeBySerialAndToken(),
              queryParameters: {'productSerial': productSerial});
      return NetworkResponse.fromResponse(response,
          converter: (json) => DigicodeCheck.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> linkDigicodeToPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.linkDigicodeToPro(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> linkDigicodeToProV2(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.linkDigicodeToProV2(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: UpdateInfoResult(status: response.statusCode == 200));
    } on DioException catch (e) {
      if (e.response?.data != null && e.response!.data['error'] is String) {
        return NetworkResponse.fromResponse(e.response!,
            value: UpdateInfoResult(
                status: false, msg: e.response!.data['error']));
      }
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> linkTerminalToProV2(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.linkTerminalToProV2(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getDigicodesLogs(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getDigicodesLogs(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => DigicodeLogModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getPlacesFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPlacesFromPro());
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => PlaceModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getPlatformsList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.getPlatformsList(),
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => PlatformModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getPlatformsListDoubleAuth(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPlatformsListDoubleAuth(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => PlatformModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> activateDoubleAuthForPlatform(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.activateDoubleAuthForPlatform(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> createPlace(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createPlace(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => PlaceModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> updatePlace(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updatePlace(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> deletePlace(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.deletePlace(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getPlaceCatByLang(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPlaceCatByLang(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => PlaceCatModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> unlockDigicode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.unlockDigicode(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> checkDigicodeOnline(digicodeId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.checkDigicodeOnline(),
              queryParameters: {'digicodeId': digicodeId});
      return NetworkResponse.fromResponse(response,
          prefix: 'status', converter: (json) => json);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> setLanguagesForPro(proLanguageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.setLanguagesForPro(),
              data: {'proLanguageCode': proLanguageCode});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> setMainLanguageForPro(proLanguageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.setMainLanguageForPro(),
              data: {'proLanguageCode': proLanguageCode});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllEventsFromPro() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAllEventsFromPro(),
              queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => EventInfoModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllServicesOfProFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAllServicesOfProFromPro(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => ProServiceModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> callNextTicket(serviceId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.callNextTicket(), data: {'serviceId': serviceId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => TicketInfoModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> cancelTicketByPro(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.cancelTicketByPro(),
              data: {'ticketUniqueId': ticketUniqueId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> canceledByProAfterCallBack(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.canceledByProAfterCallBack(),
              data: {'ticketUniqueId': ticketUniqueId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => EventInfoModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> confirmTicketByPro(
      ticketUniqueId, status, comment) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.confirmTicketByPro(), data: {
        'ticketUniqueId': ticketUniqueId,
        'confirm': status,
        'comment': comment
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => EventInfoModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createTicketByPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createTicketByPro(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TicketInfoModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllServicesOfProFromUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAllServicesOfProFromUser(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => ProServicesBusiness.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllServicesAroundMe(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAllServicesAroundMe(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => ServicesAroundMeModel.fromJson(e))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createTicketByUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createTicketByUser(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TicketInfoModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> declareUserAsAbsentFromPro(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.declareUserAsAbsentFromPro(),
              data: {'ticketUniqueId': ticketUniqueId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => EventInfoModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> deleteEventsFromPro(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.deleteEventsFromPro(), data: {
        'events': [ticketUniqueId]
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => EventInfoModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getEventInfoFromPro(eventUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getEventInfoFromPro(),
              queryParameters: {'eventUniqueId': eventUniqueId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => EventInfoModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getListTimeFramesByServiceForDay(
      dateToday, serviceId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getListTimeFramesByServiceForDay(),
              queryParameters: {
            'dateToday': dateToday,
            'serviceId': serviceId
          });
      return NetworkResponse.fromResponse(response,
          converter: (json) => ListTimeFrameModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getServiceAvailabilityForMonth(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getServiceAvailabilityForMonth(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => ServiceAvailabilityModel.fromJson(e))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> startTicket(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.startTicket(),
              data: {'ticketUniqueId': ticketUniqueId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> stopTicket(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.stopTicket(),
              data: {'ticketUniqueId': ticketUniqueId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> ticketUserNotResponse(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.ticketUserNotResponse(),
              data: {'ticketUniqueId': ticketUniqueId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateDelay(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.updateDelay(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => ServiceDelayModel.fromJson(e))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getLoginSessions(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getLoginSessions(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => LoginSession.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getLoginSessionsById(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getLoginSessionsById(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => LoginSession.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createNonExistingRandomCode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.createNonExistingRandomCode(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response, converter: (json) => json);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> revokeLoginSession(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.revokeLoginSession(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> allowPendingLoginOnPlatformByQrCode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.allowPendingLoginOnPlatformByQrCode(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> allowPendingLoginOnPlatformById(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.allowPendingLoginOnPlatformById(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> activatePourDevice(deviceId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.activatePourDevice(), data: {'deviceId': deviceId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      if (e.response?.statusCode == 400)
        return NetworkResponse.fromResponse(e.response!,
            value: e.response!.data['error']);
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createPurchaseByCardOnFileStripe(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createPurchaseByCardOnFileStripe(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PaymentInfoModel>((i) => PaymentInfoModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getBgVideo(countryCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.getBgVideo(),
              queryParameters: {'countryCode': countryCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['videoUrl']);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCurrencyAvailableForToken(tokenType) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.getCurrencyAvailableForToken(),
          queryParameters: {'tokenType': tokenType});
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List<dynamic>).map<String>((i) => i).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCurrentTokenValue(tokenType,
      {currency = 'EUR'}) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.getCurrentTokenValue(),
          queryParameters: {'tokenType': tokenType, 'currency': currency});
      return NetworkResponse.fromResponse(response,
          converter: (json) => double.parse(json.toString()));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getHistoryFromUser() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.getHistoryFromUser());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<History>((i) => History.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPmbDevicesAroundMe(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.getPmbDevicesAroundMe(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PmbDevice>((i) => PmbDevice.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getWalletInfo() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getWalletInfo());
      return NetworkResponse.fromResponse(response,
          converter: (json) => WalletInfo.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> ipAPI() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response =
          await AppClient(requiredToken: false, baseUrl: 'http://ip-api.com/')
              .get('json');
      return NetworkResponse.fromResponse(response,
          converter: (json) => IpInfo.fromJson(json), prefix: '');
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> scanDeviceQr(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.scanDeviceQr(), queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => ScanQr.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkAndAddNewContact(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .post(AppEndpoint.checkAndAddNewContact(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_addItemToPlaylist(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.v2_addItemToPlaylist(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => PlaylistItemModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_createPlaylistFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.v2_createPlaylistFromPro(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => PlaylistModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_deletePlaylistFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .delete(AppEndpoint.v2_deletePlaylistFromPro(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_getItemContentFromPlaylist(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.v2_getItemContentFromPlaylist(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PlaylistItemModel>((i) => PlaylistItemModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_getPlaylistFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .get(AppEndpoint.v2_getPlaylistFromPro(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PlaylistModel>((i) => PlaylistModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  // @override
  // Future<NetworkResponse> v2_getPlaylistsFromTerminal(params) async {
  //   bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
  //   if (isDisconnect) return NetworkResponse.withDisconnect();
  //   try {
  //     final token = await appPrefs.getNormalToken();
  //     Response response = await AppClients(requiredToken: true, token: token)
  //         .get(AppEndpoint.v2_getPlaylistsFromTerminal(),
  //             queryParameters:_mapParms(params));
  //     return NetworkResponse.fromResponse(response,
  //         converter: (json) => (json as List<dynamic>)
  //             .map<PlaylistModel>((i) => PlaylistModel.fromJson(i))
  //             .toList());
  //   } on DioException catch (e) {
  //     return NetworkResponse.withErrorRequest(e);
  //   } catch (e) {
  //     return NetworkResponse.withErrorConvert(e);
  //   }
  // }

  @override
  Future<NetworkResponse> v2_removeItemFromPlaylist(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .delete(AppEndpoint.v2_removeItemFromPlaylist(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_linkPlaylistToTerminalForPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.v2_linkPlaylistToTerminalForPro(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_addScheduleForPlaylist(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.v2_addScheduleForPlaylist(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_updateScheduleForPlaylist(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.v2_updateScheduleForPlaylist(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_removeScheduleForPlaylist(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.v2_removeScheduleForPlaylist(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_updateOrderItemsFromPlaylist(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.v2_updateOrderItemsFromPlaylist(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_getProblemTypesList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.v2_getProblemTypesList(),
              queryParameters: _mapParms(params));

      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ProblemType>((i) => ProblemType.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getFaqGroupsFromPlatformId(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getFaqGroupsFromPlatformId(),
              queryParameters: _mapParms(params));

      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<FaqGroup>((i) => FaqGroup.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> clickOnFaq(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.clickOnFaq(), queryParameters: _mapParms(params));

      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ProblemType>((i) => ProblemType.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getProblemTypesForGroupId(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getProblemTypesForGroupId(),
              queryParameters: _mapParms(params));

      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ProblemType>((i) => ProblemType.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getFaqForProblemType(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getFaqForProblemType(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<FaqItem>((i) => FaqItem.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_getMyTicketsFromCustomer(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.v2_getMyTicketsFromCustomer(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        prefix: '',
        converter: (json) => ListTicket.fromJson(json),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_postTicketByCustomer(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.v2_postTicketByCustomer(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TicketDetail.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_getMyTicketsFromAgent(ticketStatusId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.v2_getMyTicketsFromAgent(), queryParameters: {
        'languageCode': appPrefs.languageCode,
        'ticketStatusId': ticketStatusId,
      });
      return NetworkResponse.fromResponse(
        response,
        prefix: '',
        converter: (json) => ListTicket.fromJson(json),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_getAgentList() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.v2_getAgentList(), queryParameters: {
        'languageCode': appPrefs.languageCode,
      });
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json as List).map((e) => AgentModel.fromJson(e)).toList(),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_answerToTicketFromAgent(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.v2_answerToTicketFromAgent(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        value: response.statusCode == 200,
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_answerToTicketFromCustomer(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.v2_answerToTicketFromCustomer(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        value: response.statusCode == 200,
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> v2_assignTicketToAgents(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.v2_assignTicketToAgents(), data: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        value: response.statusCode == 200,
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAnswersOfTicket(ticketId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAnswersOfTicket(), queryParameters: {
        'ticketId': ticketId,
      });
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json as List).map((e) => TicketAnswer.fromJson(e)).toList(),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getLinksListFromMainLink(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          AppEndpoint.getLinksListFromMainLink(),
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        converter: (json) => MainLink.fromJson(json),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addLocationForUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addLocationForUser(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => LocationModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> createPurchaseByPaymentMethodStripe(params) {
    // TODO: implement createPurchaseByPaymentMethodStripe
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> createPurchaseByTokenStripe(params) {
    // TODO: implement createPurchaseByTokenStripe
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getAllEventsFromEventsWebsite(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAllEventsFromEventsWebsite(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => PublishEvent.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getCategoriesFromEventsWebsite(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getCategoriesFromEventsWebsite(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => PublishEventCategory.fromJson(e))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getHashtagsList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getHashtagsList(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => HashTagModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getLocationFromUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getLocationFromUser(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => LocationModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getMaincatList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getMaincatList(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => MainCatModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getResultsByCatFromEventsWebsite(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getResultsByCatFromEventsWebsite(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => PublishEvent.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getResultsFromFilters(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getResultsFromFilters(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => ProsModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getResultsFromSearch(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getResultsFromSearch(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => ProsModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getSpecList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getSpecList(), queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => SpecCatModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getSubcatList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getSubcatList(), queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => SubCatModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> removeLocationForUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.removeLocationForUser(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> setLocationDefault(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.setLocationDefault(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> updateLocationForUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateLocationForUser(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> loginTerminal(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false)
          .post(AppEndpoint.loginTerminal(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response, value: response.data);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getVideoPlaylistsFromTerminal(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getVideoPlaylistsFromTerminal(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
      );
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getServicesListFromTerminal() async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getServicesListFromTerminal());
      return NetworkResponse.fromResponse(response,
          converter: (json) => QueueOowooModel.listFromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getNextTicketAvailableForQueueFromTerminal(id) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(
        AppEndpoint.getNextTicketAvailableForQueueFromTerminal(),
        queryParameters: {'serviceId': id},
      );
      return NetworkResponse.fromResponse(response,
          converter: (json) => TicketModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> cancelNextTicketAvailableForQueueFromTerminal(
      id) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(
        AppEndpoint.cancelNextTicketAvailableForQueueFromTerminal(),
        data: {'ticketUniqueId': id},
      );
      return NetworkResponse.fromResponse(
        response,
        value: response.data,
      );
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> v2_getPlaylistsFromTerminal(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.v2_getPlaylistsFromTerminal());
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => PlaylistModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getListOfProsFromAdmin(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getListOfProsFromAdmin(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => ProInfo.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getProInfosFromSA(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getProInfosFromSA(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => ProInfo.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getUserInfosFromSA(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getUserInfosFromSA(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => UserModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getListOfRoles(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getListOfRoles(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => RolePro.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getListOfUsersFromAdmin(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getListOfUsersFromAdmin(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => UserModel.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getThumbnailUrlsForCat(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getThumbnailUrlsForCat(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => ThumbnailOfCate.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getThumbnailCat(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getThumbnailCat(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => ThumbnailOfCate.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> addThumbnailUrlForCat(
      bytes, filePath, label, catId) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      String fileName = filePath.split('/').last;
      FormData formData = FormData.fromMap({
        "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
        "label": label,
        "catId": catId,
      });
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addThumbnailUrlForCat(), data: formData);
      return NetworkResponse.fromResponse(response,
          converter: (json) => ThumbnailUrlOfCat.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> createThumbnailCat(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createThumbnailCat(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => ThumbnailUrlOfCat.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> updateThumbnailUrlForCat(
      bytes, filePath, label, id) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      String fileName = filePath.split('/').last;
      FormData formData = FormData.fromMap({
        "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
        "label": label,
        "id": id,
      });
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateThumbnailUrlForCat(), data: formData);
      return NetworkResponse.fromResponse(response,
          converter: (json) => ThumbnailUrlOfCat.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> deleteThumbnailUrlForCat(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.deleteThumbnailUrlForCat(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> setProRole(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.setProRole(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> v2GetProductCategories(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.v2GetProductCategories(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => ProductCategory.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> v2getProductRefByCatId(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.v2getProductRefByCatId(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => ProductReference.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> setKisblePublicThumbnailForPro(
      bytes, path, proId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      String fileName = path.split('/').last;
      FormData data = FormData.fromMap({
        'proId': proId,
        "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
      });
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.setKisblePublicThumbnailForPro(), data: data);
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['fileName']);
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> setThumbnailForProFromSA(bytes, path, proId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      String fileName = path.split('/').last;
      FormData data = FormData.fromMap({
        'proId': proId,
        "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
      });
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.setThumbnailForProFromSA(), data: data);
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['fileName']);
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> setThumbnailForUserFromSA(bytes, path, proId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      String fileName = path.split('/').last;
      FormData data = FormData.fromMap({
        'proId': proId,
        "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
      });
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.setThumbnailForUserFromSA(), data: data);
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['fileName']);
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCrmStatusList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getCrmStatusList(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json as List).map((e) => CrmStatus.fromJson(e)).toList(),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getListOfLeadsFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getListOfLeadsFromPro(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json as List).map((e) => CrmLead.fromJson(e)).toList(),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addCrmLeadsToProject(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addCrmLeadsToProject(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> createCrmLead(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createCrmLead(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> createCrmProjectFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createCrmProjectFromPro(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> deleteCrmLead(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.deleteCrmLead(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getCrmIntouchList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getCrmIntouchList(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json as List).map((e) => CrmIntouch.fromJson(e)).toList(),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getListCrmProjectsFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getListCrmProjectsFromPro(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json as List).map((e) => CrmProject.fromJson(e)).toList(),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> removeCrmLeadsToProject(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.removeCrmLeadsToProject(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> setThumbnailForCrmLead(bytes, path, uuid) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      String fileName = path.split('/').last;
      FormData data = FormData.fromMap({
        'uuid': uuid,
        "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
      });
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.setThumbnailForCrmLead(), data: data);
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['fileName']);
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateCrmLead(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateCrmLead(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getCrmLeadDetails(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getCrmLeadDetails(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        converter: (json) => CrmLead.fromJson(json),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createInTouchHistory(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createInTouchHistory(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getCrmLeadHistory(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getCrmLeadHistory(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json as List).map((e) => CrmIntouchHistory.fromJson(e)).toList(),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCrmProjectMemberList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getCrmProjectMemberList(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json as List).map((e) => ProInfo.fromJson(e)).toList(),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addCrmProjectMember(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addCrmProjectMember(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> removeCrmProjectMember(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.removeCrmProjectMember(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getPaymentMethodList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPaymentMethodList(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json as List).map((e) => PaymentMethod.fromJson(e)).toList(),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPaymentStatusList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPaymentStatusList(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json as List).map((e) => PaymentStatus.fromJson(e)).toList(),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOrdersFromProOrUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOrdersFromProOrUser(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json as List).map((e) => PaymentOrder.fromJson(e)).toList(),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOrdersFromAdmin(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOrdersFromAdmin(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json as List).map((e) => PaymentOrder.fromJson(e)).toList(),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateOrderDetailsByAdmin(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateOrderDetailsByAdmin(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> setHomeBookmark(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.setHomeBookmark(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateActiveForHome(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateActiveForHome(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbSearchMulti(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/search/multi',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: 'results',
          converter: (json) =>
              (json as List).map((e) => TheMovieModel.fromJson(e)).toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbSearchPerson(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/search/person',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: 'results',
          converter: (json) =>
              (json as List).map((e) => TheMoviePerson.fromJson(e)).toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbSearchMovie(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/search/movie',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: 'results',
          converter: (json) =>
              (json as List).map((e) => TheMovieModel.fromJson(e)).toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbMovieDetail(id, params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/movie/$id',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: '', converter: (json) => TheMovieDetail.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbMovieReview(id, params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/movie/$id/reviews',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: 'results',
          converter: (json) =>
              (json as List).map((e) => TheMovieReview.fromJson(e)).toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbTrending(
      media_type, time_window, params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/trending/$media_type/$time_window',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: 'results',
          converter: (json) =>
              (json as List).map((e) => TheMovieModel.fromJson(e)).toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbDiscoverMovie(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/discover/movie',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: 'results',
          converter: (json) =>
              (json as List).map((e) => TheMovieModel.fromJson(e)).toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbGenres(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/genre/movie/list',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: 'genres',
          converter: (json) => (json as List)
              .map((e) => TheMovieDetailGenres.fromJson(e))
              .toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbMovieDetailCredits(id, params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/movie/$id/credits',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: '', converter: (json) => TheMovieCredit.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbMovieDetailImages(id, params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/movie/$id/images',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: '', converter: (json) => TheMovieImage.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbMovieDetailVideos(id, params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/movie/$id/videos',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: 'results',
          converter: (json) =>
              (json as List).map((e) => TheMovieVideo.fromJson(e)).toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbPersonDetail(id, params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/person/$id',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: '', converter: (json) => TheMoviePersonDetail.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbPersonCredits(id, params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/person/$id/movie_credits',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: '', converter: (json) => TheMoviePersonCredit.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbPersonDetailImages(id, params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/person/$id/images',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: '', converter: (json) => TheMoviePersonImage.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbMovieDetailSimilar(id, params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/movie/$id/similar',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: 'results',
          converter: (json) =>
              (json as List).map((e) => TheMovieModel.fromJson(e)).toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbSearchKeyword(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/search/keyword',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: 'results',
          converter: (json) =>
              (json as List).map((e) => TheMovieKeyword.fromJson(e)).toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> themoviedbConfigLanguage(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.themoviedb.org/3/configuration/languages',
          queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          prefix: '',
          converter: (json) =>
              (json as List).map((e) => TheMovieLanguage.fromJson(e)).toList());
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createMovieCollection(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createMovieCollection(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TheMovieCollection.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addMovieBookmarksToCollection(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addMovieBookmarksToCollection(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> removeMovieFromCollection(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.removeMovieFromCollection(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addMovieInArckipelDb(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addMovieInArckipelDb(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => TmmAddToArckipelResult.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getMyBookmarks(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getMyBookmarks(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => TheMovieBookmark.fromJson(e)).toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getMyMovieCollections(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getMyMovieCollections(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => TheMovieCollection.fromJson(e))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateMovieCollection(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateMovieCollection(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateMyMovieInfos(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateMyMovieInfos(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getSubscriptionState(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getSubscriptionState(),
              queryParameters: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<SubscriptionState>((i) => SubscriptionState.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> removeGalleryPicForHome(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.removeGalleryPicForHome(),
              data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> deleteHome(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.deleteHome(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> setLogoForAgency(bytes, path, homeId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      String fileName = path.split('/').last;
      FormData data = FormData.fromMap({
        'uuid': homeId,
        "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
      });
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.setLogoForAgency(), data: data);
      return NetworkResponse.fromResponse(response,
          converter: (json) => FileUploadResult.fromJson(json));
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateAgency(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateAgency(), data: _mapParms(params));
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getBlogPosts(params) {
    // TODO: implement getBlogPosts
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getCitiesListByPurposeId(params) {
    // TODO: implement getCitiesListByPurposeId
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getCurrentFiltersAvailable(params) {
    // TODO: implement getCurrentFiltersAvailable
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getCurrentHomesOfTerminal(params) {
    // TODO: implement getCurrentHomesOfTerminal
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getHomeBookmarkList(params) {
    // TODO: implement getHomeBookmarkList
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getHomeCatList(params) {
    // TODO: implement getHomeCatList
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getHomeCitiesList(params) {
    // TODO: implement getHomeCitiesList
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getHomeDetails(params) {
    // TODO: implement getHomeDetails
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getHomePurposeTypesList(params) {
    // TODO: implement getHomePurposeTypesList
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getHomesAroundMe(params) {
    // TODO: implement getHomesAroundMe
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getHomesDetailsByListFromTerminal(params) {
    // TODO: implement getHomesDetailsByListFromTerminal
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getHomesListByFilters(params) {
    // TODO: implement getHomesListByFilters
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getHomesListForUserOrPro(params) {
    // TODO: implement getHomesListForUserOrPro
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getListOfAgencies(params) {
    // TODO: implement getListOfAgencies
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getMediaForOoh(params) {
    // TODO: implement getMediaForOoh
    throw UnimplementedError();
  }
}
